use crate::login_window::LoginWindow;

use gtk4 as gtk;
use gtk::{glib, gdk, gio, prelude::*, subclass::prelude::*};
use std::cell::RefCell;
use simplelogin as sl;

mod imp {
    use super::*;
    use glib::closure_local;
    
    #[derive(Debug, glib::Properties)]
    #[properties(wrapper_type = super::Application)]
    pub struct Application {
        #[property(name = "api-key", type = Option<String>,
            get = Self::api_key, set = Self::set_api_key, nullable)]
        pub settings: RefCell<gio::Settings>,
        pub context: RefCell<sl::Context>,
    }

    impl Application {
        fn api_key(&self) -> Option<String> {
            self.settings.borrow().value("api-key").get()
        }

        fn set_api_key(&self, value: Option<String>) {
            _ = self.settings.borrow().set("api-key", value);
        }

        fn activate_no_api_key(&self) {
            let login_window = LoginWindow::new(&self.obj());
            login_window.connect_closure(
                "logged-in", false,
                closure_local!(@weak-allow-none self as opt_this
                    => move |win: LoginWindow| {
                        win.close();
                        if let Some(this) = opt_this {
                            let obj = this.obj();
                            let ctx = obj.context().borrow();
                            let api_key = ctx.api_key();
                            this.set_api_key(api_key.clone());
                            this.activate_has_api_key();
                        }
                    }));
            login_window.present();
        }

        fn activate_has_api_key(&self) {
            println!("Logged in with API Key: {}", self.api_key().unwrap());
        }
    }

    impl Default for Application {
        fn default() -> Self {
            let settings = RefCell::new(gio::Settings::new(crate::APP_ID));
            let api_key: Option<String> = settings.borrow().value("api-key").get();
            Self {
                settings,
                context: RefCell::new(sl::Context::new(api_key.as_deref())),
            }
        }
    }

    #[glib::object_subclass]
    impl ObjectSubclass for Application {
        const NAME: &'static str = "Application";
        type Type = super::Application;
        type ParentType = gtk::Application;
    }

    impl ApplicationImpl for Application {
        fn activate(&self) {
            let css = gtk::CssProvider::new();
            gtk::style_context_add_provider_for_display(
                &gdk::Display::default().unwrap(),
                &css, gtk::STYLE_PROVIDER_PRIORITY_APPLICATION
            );
            css.load_from_string(include_str!("../data/global.css"));
            match self.api_key() {
                Some(_) => self.activate_has_api_key(),
                None => self.activate_no_api_key(),
            }
        }
    }

    #[glib::derived_properties]
    impl ObjectImpl for Application {}
    impl GtkApplicationImpl for Application {}
}

glib::wrapper! {
    pub struct Application(ObjectSubclass<imp::Application>)
        @extends gtk::Application, gio::Application,
        @implements gio::ActionMap, gio::ActionGroup;
}

impl Application {
    pub fn new(app_id: &str) -> Self {
        glib::Object::builder().property("application-id", app_id).build()
    }

    pub fn context(&self) -> &RefCell<sl::Context> {
        &self.imp().context
    }
}
